"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Math = exports.MathInlineNodeView = exports.MathBlockNodeView = exports.MathNode = void 0;
const tslib_1 = require("tslib");
const markdown_it_katex_1 = tslib_1.__importDefault(require("markdown-it-katex"));
const prosemirror_commands_1 = require("prosemirror-commands");
const prosemirror_utils_1 = require("prosemirror-utils");
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_inputrules_1 = require("prosemirror-inputrules");
const selection_1 = require("../../../utils/selection");
const inputrules_1 = require("../../../utils/inputrules");
const view_and_edit_1 = require("./view-and-edit");
const const_1 = require("./const");
const commands_1 = require("./commands");
var const_2 = require("./const");
Object.defineProperty(exports, "MathNode", { enumerable: true, get: function () { return const_2.MathNode; } });
var view_and_edit_2 = require("./view-and-edit");
Object.defineProperty(exports, "MathBlockNodeView", { enumerable: true, get: function () { return view_and_edit_2.MathBlockNodeView; } });
Object.defineProperty(exports, "MathInlineNodeView", { enumerable: true, get: function () { return view_and_edit_2.MathInlineNodeView; } });
const mathIAction = 'addMathInline';
const mathBAction = 'toMathBlock';
const mathITemplate = 'f(x)=';
const Math = (builder) => {
    builder.configureMd((md) => md.use(markdown_it_katex_1.default));
    builder
        .addNode(const_1.MathNode.Inline, () => ({
        spec: {
            group: 'inline math',
            content: 'text*',
            inline: true,
            marks: '',
            code: true,
            toDOM: () => [
                'span',
                { class: const_1.CLASSNAMES.Inline.Container },
                ['span', { class: const_1.CLASSNAMES.Inline.Sharp, contenteditable: 'false' }, '$'],
                ['span', { class: const_1.CLASSNAMES.Inline.Content }, 0],
                ['span', { class: const_1.CLASSNAMES.Inline.Sharp, contenteditable: 'false' }, '$'],
            ],
            parseDOM: [{ tag: `span.${const_1.CLASSNAMES.Inline.Content}`, priority: 200 }],
        },
        fromYfm: {
            tokenName: 'math_inline',
            tokenSpec: { name: const_1.MathNode.Inline, type: 'block', noCloseToken: true },
        },
        toYfm: (state, node) => {
            state.text(`$${node.textContent}$`, false);
            state.closeBlock();
        },
    }))
        .addNode(const_1.MathNode.Block, () => ({
        spec: {
            group: 'block math',
            content: 'text*',
            marks: '',
            code: true,
            toDOM: () => ['div', { class: 'math-block' }, 0],
            parseDOM: [{ tag: 'div.math-block', priority: 200 }],
        },
        fromYfm: {
            tokenName: 'math_block',
            tokenSpec: { name: const_1.MathNode.Block, type: 'block', noCloseToken: true },
        },
        toYfm: (state, node) => {
            state.text(`$$${node.textContent}$$\n\n`, false);
            state.closeBlock();
        },
    }));
    builder.addKeymap(() => ({
        Enter: commands_1.ignoreIfCursorInsideMathInline,
        Backspace: (0, prosemirror_commands_1.chainCommands)(commands_1.moveCursorToEndOfMathInline, commands_1.removeEmptyMathInlineIfCursorIsAtBeginning),
    }));
    builder.addPlugin(view_and_edit_1.mathViewAndEditPlugin).addInputRules((deps) => ({
        rules: [
            (0, prosemirror_inputrules_1.textblockTypeInputRule)(/^\$\$\s$/, (0, const_1.mathBType)(deps.schema)),
            (0, inputrules_1.inlineNodeInputRule)(/\$[^$\s]+\$$/, (match) => (0, const_1.mathIType)(deps.schema).create(null, deps.schema.text(match.replace(/\$/g, '')))),
        ],
    }));
    builder
        .addAction(mathIAction, (deps) => {
        const type = (0, const_1.mathIType)(deps.schema);
        const cmd = (state, dispatch) => {
            const { selection } = state;
            if ((0, selection_1.isTextSelection)(selection) && selection.$from.sameParent(selection.$to)) {
                if (dispatch) {
                    const fragment = selection.content().content;
                    const selectedText = fragment.textBetween(0, fragment.size) || mathITemplate;
                    const tr = state.tr;
                    selection.replaceWith(tr, type.create(null, deps.schema.text(selectedText)));
                    tr.setSelection(prosemirror_state_1.TextSelection.create(tr.doc, selection.from + selectedText.length + 1));
                    dispatch(tr);
                }
                return true;
            }
            return false;
        };
        return {
            isActive: (state) => (0, prosemirror_utils_1.hasParentNodeOfType)(type)(state.selection),
            isEnable: cmd,
            run: cmd,
        };
    })
        .addAction(mathBAction, (deps) => {
        const type = (0, const_1.mathBType)(deps.schema);
        const cmd = (0, prosemirror_commands_1.setBlockType)(type);
        return {
            isActive: (state) => (0, prosemirror_utils_1.hasParentNodeOfType)(type)(state.selection),
            isEnable: cmd,
            run: cmd,
        };
    });
};
exports.Math = Math;
