"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.YfmTabs = void 0;
const tslib_1 = require("tslib");
const log_1 = tslib_1.__importDefault(require("@doc-tools/transform/lib/log"));
const tabs_1 = tslib_1.__importDefault(require("@doc-tools/transform/lib/plugins/tabs"));
const toYfm_1 = require("./toYfm");
const const_1 = require("./const");
const fromYfm_1 = require("./fromYfm");
const spec_1 = require("./spec");
const plugins_1 = require("./plugins");
const prosemirror_commands_1 = require("prosemirror-commands");
const ignoreMutation = (node, view, getPos) => (mutation) => {
    if (mutation instanceof MutationRecord &&
        mutation.type === 'attributes' &&
        mutation.attributeName) {
        const newAttr = mutation.target.getAttribute(mutation.attributeName);
        view.dispatch(view.state.tr.setNodeMarkup(getPos(), null, Object.assign(Object.assign({}, node.attrs), { [mutation.attributeName]: String(newAttr) })));
        return true;
    }
    return false;
};
const YfmTabs = (builder) => {
    builder
        .configureMd((md) => md.use(tabs_1.default, { log: log_1.default }))
        .addNode(const_1.TabsNode.Tab, () => ({
        spec: spec_1.spec[const_1.TabsNode.Tab],
        toYfm: toYfm_1.toYfm[const_1.TabsNode.Tab],
        fromYfm: {
            tokenSpec: fromYfm_1.fromYfm[const_1.TabsNode.Tab],
            tokenName: 'tab',
        },
        // FIX: ignore mutation and don't rerender node when yfm.js switch tab
        // @ts-expect-error
        view: () => (node, view, getPos) => ({
            ignoreMutation: ignoreMutation(node, view, getPos),
        }),
    }))
        .addNode(const_1.TabsNode.TabsList, () => ({
        spec: spec_1.spec[const_1.TabsNode.TabsList],
        toYfm: toYfm_1.toYfm[const_1.TabsNode.TabsList],
        fromYfm: {
            tokenSpec: fromYfm_1.fromYfm[const_1.TabsNode.TabsList],
            tokenName: 'tab-list',
        },
    }))
        .addNode(const_1.TabsNode.TabPanel, () => ({
        spec: spec_1.spec[const_1.TabsNode.TabPanel],
        toYfm: toYfm_1.toYfm[const_1.TabsNode.TabPanel],
        fromYfm: {
            tokenSpec: fromYfm_1.fromYfm[const_1.TabsNode.TabPanel],
            tokenName: 'tab-panel',
        },
        // FIX: ignore mutation and don't rerender node when yfm.js switch tab
        // @ts-expect-error
        view: () => (node, view, getPos) => ({
            ignoreMutation: ignoreMutation(node, view, getPos),
        }),
    }))
        .addNode(const_1.TabsNode.Tabs, () => ({
        spec: spec_1.spec[const_1.TabsNode.Tabs],
        toYfm: toYfm_1.toYfm[const_1.TabsNode.Tabs],
        fromYfm: {
            tokenSpec: fromYfm_1.fromYfm[const_1.TabsNode.Tabs],
            tokenName: 'tabs',
        },
    }))
        .addKeymap(() => ({
        Backspace: (0, prosemirror_commands_1.chainCommands)(plugins_1.tabPanelBackspace, plugins_1.tabBackspace),
    }));
};
exports.YfmTabs = YfmTabs;
